#include <cassert>
#include <chrono>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <string>

#include "charmap.h"

// ./generator <a> <m> <pixel graphics output file name> <vector graphics output file name>
//
// size a, default a = 256, the figure size is n = a^2, with a pixels in x direction and a pixels in y direction
// number of disks m used to generate the image, default m = 10
//
// default pixel output file name: benchmark.pxl
// default vector output file name: benchmark.vct
//
int main(int argc, char** argv)
{
   int a = 256;
   if(argc > 1) a = std::atoi(argv[1]);
   assert(a > 1);
   std::cout << "Image edge size:\t" << a << " pixels (" << a << " x " << a << ")\n";
   
   int m = 10;
   if(argc > 2) m = std::atoi(argv[2]);
   assert(m > 0);
   std::cout << "No. random disks:\t" << m << "\n";
   
   std::string pixout_fname = "benchmark.pxl";
   if(argc > 3) pixout_fname = argv[3];
   std::cout << "Pixel output to:\t" << pixout_fname << "\n";
   
   std::string vctout_fname = "benchmark.vct";
   if(argc > 4) vctout_fname = argv[4];
   std::cout << "Vector output to:\t" << vctout_fname << "\n";

   // random seed from chrono
   std::srand(
      std::chrono::duration_cast<std::chrono::nanoseconds>(
         std::chrono::high_resolution_clock::now().time_since_epoch()
      ).count()
   );

   // create the random figure
   //
   diskgraphics::DiskVector dv;
   dv.set_size(a, a);
   dv.set_background(255);
   float disk_radius = a;
   unsigned char disk_colour = 0;
   for(int i = 0; i < m; i++)
   {
      diskgraphics::Disk dsk;
      disk_radius /= 1.125 + 0.125*std::rand()/(float)RAND_MAX;  // shrink down by random factor between 1.125 and 1.25
      
      dsk.x = a * (0.75*(std::rand()/(float)RAND_MAX) + 0.125);
      dsk.y = a * (0.75*(std::rand()/(float)RAND_MAX) + 0.125);
      dsk.r = disk_radius;
      dsk.colour = disk_colour;
      dv.add_disk(dsk);
      
      // switch colour from one disk to the next with 2/3 probability
      if(std::rand() > RAND_MAX/3.0) disk_colour = 255 - disk_colour;
   }
   
   // vector output
   std::ofstream vctostrm(vctout_fname);
   if(!vctostrm) return EXIT_FAILURE;
   vctostrm << dv;
   vctostrm.close();
   
   // pixel conversion and output
   //
   diskgraphics::Charmap cm(dv);
   if(32 >= a) cm.out(&std::cout, true);  // if a small figure, also show on cout
   std::ofstream pixostrm(pixout_fname);
   if(!pixostrm) return EXIT_FAILURE;
   pixostrm << cm;
   pixostrm.close();
}
